from django.urls import path
from .views import *

app_name = 'accounts'

urlpatterns = [
    path('register/', RegisterView.as_view(), name='register'),
    path('login/', CustomLoginView.as_view(), name='login'),
    path('logout/', CustomLogoutView.as_view(), name='logout'),
    path('password_change/', CustomPasswordChangeView.as_view(), name='password_change'),
    path('password_change/done/', CustomPasswordChangeDoneView.as_view(), name='password_change_done'),
    path('password_reset/', CustomPasswordResetView.as_view(), name='password_reset'),
    path('password_reset/done/', CustomPasswordResetDoneView.as_view(), name='password_reset_done'),
    path('reset/<uidb64>/<token>/', CustomPasswordResetConfirmView.as_view(), name='password_reset_confirm'),
    path('reset/done/', CustomPasswordResetCompleteView.as_view(), name='password_reset_complete'),
]



import argparse
import os
import re
from pathlib import Path


class DynamicCompose:
    _COMPOSE_NAME = 'compose.yml'
    _DOCKERFILE_NAME = 'Dockerfile'
    SAFELY_COVER_MARK = ('# This file was automatically generated by DynamicCompose, and can be safely deleted, '
                         'but do not edit it or remove this comment.\n')

    def __init__(self, cwd: str | Path = None, show_command: bool = True, force_cover: bool = False):
        self.show_command = show_command
        self.force_cover = force_cover
        self.cwd = cwd or Path().cwd()

        self.env_file = self.cwd / '.env'
        self.compose_dir = self.cwd / '.compose'
        self.compose_dir.mkdir(exist_ok=True)

        self.compose_content, self.dockerfile_content = '', ''
        self.compose_name, self.dockerfile_name = '', ''

        self.env = self._get_env()
        args, self.extra_args = self._get_args()
        self.env.update(dict(args.e or ()))

    @staticmethod
    def _parse_key_value(arg_string: str):
        key, value = arg_string.strip().split('=', maxsplit=1)
        return key, value

    def _get_env(self):
        env_dict = {}
        if self.env_file.exists():
            with open(self.env_file) as f:
                for line in f.readlines():
                    if line.strip():
                        key, value = self._parse_key_value(line)
                        env_dict[key] = value
        return env_dict

    def _get_args(self) -> tuple[argparse.Namespace, list[str]]:
        parser = argparse.ArgumentParser(description='Dynamic generating the Dockerfile and the compose.yml')
        parser.add_argument('-e', action='append', type=self._parse_key_value,
                            help='The environment variables are used in generating.')
        parsed_args, parsed_extra_args = parser.parse_known_args()
        return parsed_args, parsed_extra_args

    @staticmethod
    def _convert_env_var_ref(template: str) -> str:
        """Replace { to {{, {{ will be seen as literal { in python's f-string"""
        return re.sub(r'\$\{(.+?)}', r'${{\1}}', template)

    def _format(self, template: str, template_file: str | Path, name: str) -> 'DynamicCompose':
        if not template:
            path = Path(template_file or self.compose_dir / name)
            if name == self._COMPOSE_NAME:
                self.compose_name = path.name
            else:
                self.dockerfile_name = path.name
            template = path.read_text(encoding='utf-8')
        if name == self._COMPOSE_NAME:
            self.compose_content = self._convert_env_var_ref(template).format(**self.env)
        else:
            self.dockerfile_content = self._convert_env_var_ref(template).format(**self.env)
        return self

    def format_compose(self, template: str = None, template_file: str | Path = None) -> 'DynamicCompose':
        return self._format(template, template_file, self._COMPOSE_NAME)

    def format_dockerfile(self, template: str = None, template_file: str | Path = None) -> 'DynamicCompose':
        return self._format(template, template_file, self._DOCKERFILE_NAME)

    def _dump(self, target_file: str | Path, content: str) -> 'DynamicCompose':
        if content == self.compose_content:
            name = self._COMPOSE_NAME if not self.compose_name else self.compose_name
            self.compose_name = ''  # clear the name
        else:
            name = self._DOCKERFILE_NAME if not self.dockerfile_name else self.dockerfile_name
            self.dockerfile_name = ''  # clear the name

        path = Path(target_file or self.cwd / name)
        if path.exists():
            old_content = path.read_text(encoding='utf-8').strip()
            if not old_content.startswith(self.SAFELY_COVER_MARK):
                raise ValueError(f'"{path}" may not have been generated by this program.'
                                 f'You should check and delete it manually then re-run this program.')
        path.write_text(self.SAFELY_COVER_MARK + content, encoding='utf-8')
        return self

    def dump_compose(self, target_file: str | Path = None) -> 'DynamicCompose':
        return self._dump(target_file, self.compose_content)

    def dump_dockerfile(self, target_file: str | Path = None) -> 'DynamicCompose':
        return self._dump(target_file, self.dockerfile_content)

    def fd_compose(self, template: str = None, template_file: str | Path = None,
                   target_file: str | Path = None) -> 'DynamicCompose':
        self.format_compose(template, template_file)
        self.dump_compose(target_file)
        return self

    def fd_dockerfile(self, template: str = None, template_file: str | Path = None,
                      target_file: str | Path = None) -> 'DynamicCompose':
        self.format_dockerfile(template, template_file)
        self.dump_dockerfile(target_file)
        return self

    def fd_all(self) -> 'DynamicCompose':
        self.fd_compose()
        dockerfiles = [self.compose_dir / f for f in os.listdir(self.compose_dir)
                       if f.startswith(self._DOCKERFILE_NAME)]
        for dockerfile in dockerfiles:
            self.fd_dockerfile(template_file=dockerfile)
        return self

    def up(self, with_extra_args: bool = True) -> int:
        command = f'docker compose up'
        if with_extra_args:
            command = f'{command} {" ".join(self.extra_args)}'
        if self.show_command:
            print(command)
        return os.system(command)



import asyncio
import functools
import json
import datetime
import multiprocessing
import queue
import sys
import threading
import time
import traceback
from dataclasses import dataclass, asdict
from enum import Enum
from typing import Any, Callable, Union
import requests
from aiohttp import web
from psplpy.serialization_utils import Serializer


class MpHttpError(Exception):
    def __init__(self, *args, traceback_info: str = ''):
        super().__init__(*args)
        self.traceback_info = traceback_info

    def __str__(self):
        return self.traceback_info

    __repr__ = __str__


class MpHttpEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Exception):
            exception_name = type(obj).__name__
            exception_message = str(obj)
            result = f"{exception_name}: {exception_message}"
            return result
        return super().default(obj)


@dataclass
class _SubTask:
    task_id: int
    sub_task_id: int
    data: Any
    result: Any = None


class _TaskResult:
    def __init__(self, task_num: int):
        self.task_num = task_num
        self.finished_task_num = 0
        self.finished_task_list = [None] * task_num  # [None] * 0 = []
        self.is_finished = asyncio.Event()
        self.finished_time = None
        if self.task_num == 0:
            self._set_finished()

    def _set_finished(self):
        self.is_finished.set()
        self.finished_time = time.time()

    def add_finished_subtask(self, sub_task: _SubTask) -> None:
        self.finished_task_list[sub_task.sub_task_id] = sub_task.result
        self.finished_task_num += 1
        if self.finished_task_num == self.task_num:
            self._set_finished()


class MpHttpDataType(Enum):
    JSON = 'j'
    PICKLE = 'p'


@dataclass
class MpKw:
    host: str = '0.0.0.0'
    port: int = 8000
    workers: int = 1
    worker_threads: int = 1
    show_info: bool = True
    max_fetch_timeout: float = 3600
    result_timeout: float = 3600
    recv_types: tuple[MpHttpDataType] = (MpHttpDataType.JSON, MpHttpDataType.PICKLE)


class MpHttpServer(MpKw):
    _POLL_INTERVAL = 0.1

    def __init__(self, kw: MpKw):
        self.__dict__.update(asdict(kw))
        self._check_params()

        self._closed_flag = multiprocessing.Value('b', False)
        self._load = multiprocessing.Value('i', 0)
        self._ahttp_server = _AsyncHttpServer(self)

    def _check_params(self) -> None:
        if self.workers < 1 or self.worker_threads < 1:
            raise ValueError('Number of workers and worker_threads must be 1 or greater than 1')
        self.workers, self.worker_threads = int(self.workers), int(self.worker_threads)

    @staticmethod
    def _get_subtask(func: Callable):
        def wrapper(self, que: multiprocessing.Queue, *args, **kwargs) -> None:
            self.init()
            while True:
                try:
                    sub_task: _SubTask = que.get(timeout=MpHttpServer._POLL_INTERVAL)
                except queue.Empty:
                    if self._closed_flag.value:
                        break
                    continue
                func(self, sub_task, *args, **kwargs)

        return wrapper

    def init(self) -> None:
        ...

    def main_loop(self, data: Any) -> Any:
        return data

    def _main_process(self, req_que: multiprocessing.Queue, result_que: multiprocessing.Queue) -> None:
        @MpHttpServer._get_subtask
        def _process(self, sub_task: Union[multiprocessing.Queue, _SubTask]) -> None:
            with self._load.get_lock():
                self._load.value += 1
            try:
                result = self.main_loop(sub_task.data)
                sub_task.result = result
            except Exception:
                sub_task.result = MpHttpError(traceback_info=traceback.format_exc())
            finally:
                del sub_task.data
                result_que.put(sub_task)
                with self._load.get_lock():
                    self._load.value -= 1

        for _ in range(self.worker_threads):
            threading.Thread(target=_process, args=(self, req_que)).start()

    def run_server(self, new_thread: bool = False) -> 'MpHttpServer':
        for _ in range(self.workers):
            multiprocessing.Process(target=self._main_process, args=(self._ahttp_server._req_que,
                                                                     self._ahttp_server._result_que)).start()
        if self.show_info:
            sys.stderr.write(f"Starting server on ({self.host}, {self.port})...\n")
        if new_thread:
            threading.Thread(target=self._ahttp_server.run).start()
        else:
            self._ahttp_server.run()
        return self

    def close_server(self) -> None:
        self._closed_flag.value = True


class MpHttpPath(Enum):
    SUBMIT = '/submit'
    FETCH = '/fetch'
    LOAD = '/load'
    PROGRESS = '/progress'


class _AsyncHttpServer:
    def __init__(self, s: MpHttpServer):
        self._s = s
        self._serializer = Serializer()
        self._task_id = 0
        self._task_id_lock = asyncio.Lock()
        self._result_dict_lock = asyncio.Lock()
        self._result_dict: dict[int, _TaskResult] = {}
        self._req_que = multiprocessing.Queue()
        self._result_que = multiprocessing.Queue()

        self._app = web.Application()
        self._setup_routes()

    def _setup_routes(self):
        self._app.router.add_post(MpHttpPath.SUBMIT.value, self._handle_submit)
        self._app.router.add_get(MpHttpPath.FETCH.value, self._handle_fetch)
        self._app.router.add_get(MpHttpPath.LOAD.value, self._handle_load)
        self._app.router.add_get(MpHttpPath.PROGRESS.value, self._handle_process)

    async def _put_data(self, data: Any) -> int:
        sub_task_id = 0
        async with self._task_id_lock:
            task_id = self._task_id
            self._task_id += 1
        # create the TaskResult obj first, then put the SubTask, otherwise no obj to store the SubTask result
        async with self._result_dict_lock:
            self._result_dict[task_id] = _TaskResult(len(data))
        for sub_data in data:
            self._req_que.put(_SubTask(task_id, sub_task_id, sub_data))
            sub_task_id += 1
        return task_id

    async def _fetch_result(self, task_id: int, timeout: float, **kwargs) -> list[Any] | TimeoutError | KeyError:
        timeout = timeout if timeout <= self._s.max_fetch_timeout else self._s.max_fetch_timeout
        async with self._result_dict_lock:
            task = self._result_dict.get(task_id)
        if task:
            try:
                await asyncio.wait_for(task.is_finished.wait(), timeout=timeout)
            except asyncio.TimeoutError:
                return TimeoutError(f'Fetch task {task_id} result timeout after {timeout}s')
            else:
                async with self._result_dict_lock:
                    return self._result_dict.pop(task_id).finished_task_list
        return KeyError(f'Task {task_id} result not exist or expired')

    async def _result_distribution(self):
        while True:
            try:
                sub_task: _SubTask = self._result_que.get(block=False)
            except queue.Empty:
                if self._s._closed_flag.value:
                    break
                await asyncio.sleep(self._s._POLL_INTERVAL)
            else:
                async with self._result_dict_lock:
                    self._result_dict[sub_task.task_id].add_finished_subtask(sub_task)

    async def _cleanup_expired_results(self):
        last_cleanup_time = time.time()
        while True:
            if time.time() - last_cleanup_time > self._s.result_timeout / 10:
                expired_task_ids = []
                for task_id, task in self._result_dict.items():
                    if task.is_finished.is_set() and time.time() - task.finished_time > self._s.result_timeout:
                        expired_task_ids.append(task_id)
                async with self._result_dict_lock:
                    for task_id in expired_task_ids:
                        self._result_dict.pop(task_id)
                if self._s.show_info and expired_task_ids:
                    time_str = datetime.datetime.now().strftime("[%d/%b/%Y %H:%M:%S]")
                    sys.stderr.write(f'Warning - {time_str} - Result of tasks {expired_task_ids} has expired\n')
                last_cleanup_time = time.time()
            else:
                if self._s._closed_flag.value:
                    break
                await asyncio.sleep(self._s._POLL_INTERVAL)

    @staticmethod
    def _exception_handler(func):
        @functools.wraps(func)
        async def wrapper(self, request: web.Request):
            try:
                response = await func(self, request)
            except Exception:
                traceback.print_exc()
                response = web.Response(text='Internal Server Error', status=500)
            if self._s.show_info:
                current_time = datetime.datetime.now().strftime('%d/%b/%Y %H:%M:%S.%f')
                log_message = (f'-{request.remote}- [{current_time}] {request.method} {request.path} '
                               f'{request.content_length} {response.status} {len(response.body)} \n')
                sys.stderr.write(log_message)
            return response

        return wrapper

    def _return_by_data_type(self, result: Any, data_type: str, status: int = 200) -> web.Response:
        if data_type == MpHttpDataType.PICKLE.value:
            return web.Response(body=self._serializer.dump_pickle(result), status=status,
                                content_type='application/octet-stream')
        else:
            return web.Response(text=json.dumps(result, cls=MpHttpEncoder), status=status,
                                content_type='application/json')

    @_exception_handler
    async def _handle_submit(self, request: web.Request):
        content_type = request.content_type
        data_type = MpHttpDataType.JSON.value
        if content_type == 'application/octet-stream':
            data_type = MpHttpDataType.PICKLE.value

        if content_type == 'application/json' and MpHttpDataType.JSON in self._s.recv_types:
            data: Any | list[Any] = await request.json()
            if not isinstance(data, list):
                data = [data]
        elif content_type == 'application/octet-stream' and MpHttpDataType.PICKLE in self._s.recv_types:
            data: bytes = await request.read()
            data: Any = self._serializer.load_pickle(data)
        else:
            return self._return_by_data_type(ValueError("Unsupported Content-Type"), data_type, 415)
        task_id = await self._put_data(data)
        return self._return_by_data_type(task_id, data_type)

    def _process_params(self, request: web.Request, required_params: tuple = ('task_id')
                        ) -> tuple[dict | None, web.Response | None]:
        params = dict(request.query)
        if not params.get('data_type'):
            params['data_type'] = MpHttpDataType.JSON.value

        if 'task_id' in required_params:
            if params.get('task_id') is None:
                err_info = "Missing required parameter: 'id'"
                return None, self._return_by_data_type(ValueError(err_info), params['data_type'], 400)
            else:
                try:
                    params['task_id'] = int(params['task_id'])
                except ValueError:
                    err_info = "Invalid parameter type for 'task_id'. Expected an integer."
                    return None, self._return_by_data_type(ValueError(err_info), params['data_type'], 400)
        if params.get('timeout'):
            try:
                params['timeout'] = float(params['timeout'])
            except ValueError:
                err_info = "Invalid parameter type for 'timeout'. Expected a float."
                return None, self._return_by_data_type(ValueError(err_info), params['data_type'], 400)
        else:
            params['timeout'] = self._s.max_fetch_timeout
        return params, None

    @_exception_handler
    async def _handle_fetch(self, request: web.Request):
        params, response = self._process_params(request)
        if response:
            return response
        result = await self._fetch_result(**params)
        return self._return_by_data_type(result, params['data_type'])

    @_exception_handler
    async def _handle_load(self, request: web.Request):
        params, response = self._process_params(request, required_params=())
        load = self._s._load.value / (self._s.workers * self._s.worker_threads)
        return self._return_by_data_type(load, params['data_type'])

    @_exception_handler
    async def _handle_process(self, request: web.Request):
        params, response = self._process_params(request)
        if response:
            return response
        async with self._result_dict_lock:
            task = self._result_dict.get(params['task_id'])
        if task is None:
            return self._return_by_data_type(ValueError(f"No task with task_id '{params['task_id']}'"),
                                             params['data_type'], 404)
        return self._return_by_data_type((task.finished_task_num, task.task_num), params['data_type'])

    async def _start_server(self):
        runner = web.AppRunner(self._app)
        await runner.setup()
        site = web.TCPSite(runner, self._s.host, self._s.port)
        await site.start()

        task_distribution = asyncio.create_task(self._result_distribution())
        task_clean = asyncio.create_task(self._cleanup_expired_results())
        await task_distribution
        await task_clean

        await runner.cleanup()
        if self._s.show_info:
            sys.stderr.write(f'Server on ({self._s.host}, {self._s.port}) stopped.\n')

    def run(self):
        asyncio.run(self._start_server())


class MpHttpClient:
    def __init__(self, host: str = '127.0.0.1', port: int = 8080, server: MpHttpServer = None):
        self.host, self.port = host, port
        if server:
            self.host, self.port = server.host if server.host != '0.0.0.0' else '127.0.0.1', server.port

        self._loader = Serializer().load_pickle
        self._dumper = Serializer().dump_pickle
        self._data_type = MpHttpDataType.PICKLE.value
        self._headers = {'Content-Type': 'application/octet-stream'}

    def _resp(self, resp: requests.Response) -> Any:
        if resp.status_code != 200:
            raise ValueError(f"Request failed with status '{resp.status_code}', message: '{resp.text}'")
        return self._loader(resp.content)

    def _post(self, data: Any, path: str) -> Any:
        dumped_data = self._dumper(data)
        resp = requests.post(f'http://{self.host}:{self.port}{path}', data=dumped_data, headers=self._headers)
        return self._resp(resp)

    def _get(self, params: dict, path: str) -> Any:
        params['data_type'] = self._data_type
        resp = requests.get(f'http://{self.host}:{self.port}{path}', params=params, headers=self._headers)
        return self._resp(resp)

    def submit(self, data_list: list | tuple) -> int:
        """Getting the task_id for fetching data from the server"""
        return self._post(data_list, MpHttpPath.SUBMIT.value)

    def fetch(self, task_id: int, timeout: float = 3600) -> list[Any]:
        """The max timeout time depends on the setting of the server"""
        result = self._get({'task_id': task_id, 'timeout': timeout}, MpHttpPath.FETCH.value)
        if isinstance(result, (TimeoutError, KeyError)):
            raise result
        return result

    def batch(self, data_list: list | tuple, timeout: float = 3600) -> list[Any]:
        return self.fetch(self.submit(data_list), timeout)

    def get(self, data: Any, timeout: float = 3600) -> Any:
        return self.batch([data], timeout)[0]

    def load(self) -> float:
        return self._get({}, MpHttpPath.LOAD.value)

    def progress(self, task_id: int) -> tuple[int, int]:
        """return the tuple (finished_task_num, total_task_num)"""
        return tuple(self._get({'task_id': task_id}, MpHttpPath.PROGRESS.value))


class MpHttpClientJson(MpHttpClient):
    def __init__(self, host: str = '127.0.0.1', port: int = 8080, server: MpHttpServer = None):
        super().__init__(host, port, server)

        self._loader = json.loads
        self._dumper = json.dumps
        self._data_type = MpHttpDataType.JSON.value
        self._headers = {'Content-Type': 'application/json'}



import ast
import json
import os
import re
from pathlib import Path
from typing import Any, Callable


class SettingsPattern:
    LIST_MULTI = r"({name}\s*=\s*\[\s*.*?\n]\s*?$)"
    DICT_MULTI = r"({name}\s*=\s*\{{\s*.*?\n\}}\s*?$)"
    LIST_SINGLE = r"({name}\s*=\s*\[\s*.*?]\s*?$)"
    DICT_SINGLE = r"({name}\s*=\s*\{{\s*.*?\}}\s*?$)"
    LINE = r"({name}\s*=\s*.*?$)"
    VAR_DEFINE = re.compile(r'[a-zA-Z_][a-zA-Z0-9_]*\s*=\s*')

    def __init__(self, settings_str: list[str], show_info: bool = True):
        self.settings_str = settings_str
        self.show_info = show_info

    def _show_info(self, info: str) -> None:
        if self.show_info:
            print(info)

    @staticmethod
    def _format(name: str) -> list[re.Pattern]:
        # the order of the patterns is important
        pattern_list = [re.compile(SettingsPattern.LIST_MULTI.format(name=name), flags=re.DOTALL | re.MULTILINE),
                        re.compile(SettingsPattern.DICT_MULTI.format(name=name), flags=re.DOTALL | re.MULTILINE),
                        re.compile(SettingsPattern.LIST_SINGLE.format(name=name), flags=re.DOTALL | re.MULTILINE),
                        re.compile(SettingsPattern.DICT_SINGLE.format(name=name), flags=re.DOTALL | re.MULTILINE),
                        re.compile(SettingsPattern.LINE.format(name=name), flags=re.DOTALL | re.MULTILINE)]
        return pattern_list

    @staticmethod
    def _only_one_var(matched_str: str) -> bool:
        matches = SettingsPattern.VAR_DEFINE.findall(matched_str)
        if len(matches) == 1:
            return True
        return False

    @staticmethod
    def dump(name: str, value: Any) -> str:
        if isinstance(value, bool):
            return f'{name} = {str(value)}'
        return f'{name} = {json.dumps(value, ensure_ascii=False, indent=4)}'

    def _search(self, name: str, func: Callable, **kwargs) -> str:
        for pattern in self._format(name):
            if match := pattern.search(self.settings_str[0]):
                if self._only_one_var(match.group()):
                    return func(pattern, match, self.settings_str[0], name, **kwargs)
        else:
            raise KeyError(f'No matches for "{name}"')

    def search(self, name: str) -> str:
        def do(pattern: re.Pattern, match: re.Match, settings_str: str, name: str, **kwargs) -> str:
            return match.group()

        return self._search(name, do)

    def sub(self, name: str, s: str, not_in: str = '') -> None:
        def do(pattern: re.Pattern, match: re.Match, settings_str: str, name: str, **kwargs) -> str:
            if kwargs['not_in'] not in settings_str:
                return pattern.sub(kwargs['s'], settings_str)
            return settings_str

        if not not_in:
            not_in = s
        result = self._search(name, do, s=s, not_in=not_in)
        if result != self.settings_str[0]:
            self._show_info(f'Sub to {s}')
            self.settings_str[0] = result

    def add_before(self, target: str, s: str, blank_lines: int = 1) -> None:
        blank = '\n' * blank_lines
        self.sub(target, f'{s}\n{blank}\\1', not_in=s)

    def add_after(self, target: str, s: str, blank_lines: int = 1) -> None:
        blank = '\n' * blank_lines
        self.sub(target, f'\\1{blank}\n{s}', not_in=s)


class Settings:
    MIDDLEWARE_ORDER = [
        "django.middleware.security.SecurityMiddleware",
        "django.middleware.cache.UpdateCacheMiddleware",
        "django.middleware.gzip.GZipMiddleware",
        "django.middleware.http.ConditionalGetMiddleware",
        "django.contrib.sessions.middleware.SessionMiddleware",
        "django.middleware.locale.LocaleMiddleware",
        "django.middleware.common.CommonMiddleware",
        "django.middleware.csrf.CsrfViewMiddleware",
        "django.contrib.auth.middleware.AuthenticationMiddleware",
        "django.contrib.auth.middleware.LoginRequiredMiddleware",  # New in Django 5.1
        "django.contrib.messages.middleware.MessageMiddleware",
        "django.middleware.cache.FetchFromCacheMiddleware",
        "django.contrib.flatpages.middleware.FlatpageFallbackMiddleware",
        "django.middleware.http.RedirectFallbackMiddleware"
    ]

    def __init__(self, project_dir: str | Path = None, show_info: bool = True):
        self.project_dir = Path(project_dir) if project_dir else Path().cwd()
        os.chdir(self.project_dir)

        self.manage_py_path = self._find_manage_py()
        self.project_name = self._extract_project_name()
        self.project_app_dir = self.project_dir / self.project_name
        self.settings_py_path = self.project_app_dir / 'settings.py'
        self._settings_str = [self.settings_py_path.read_text(encoding='utf-8')]
        self.settings_const = self._parse_vars_from_code()
        self.sp = SettingsPattern(self._settings_str, show_info)

    def _find_manage_py(self) -> Path:
        manage_py_path = self.project_dir / 'manage.py'
        if manage_py_path.exists():
            return manage_py_path
        raise FileNotFoundError(f'manage.py not found at {manage_py_path}')

    def _extract_project_name(self) -> str:
        content = self.manage_py_path.read_text(encoding='utf-8')
        match = re.search(r"os\.environ\.setdefault\('DJANGO_SETTINGS_MODULE', '([^']+)'\)", content)
        if match:
            return match.group(1).split('.')[0]
        raise ValueError("Cannot extract project name from manage.py")

    def _parse_vars_from_code(self) -> dict:
        variables = {}

        class VariableVisitor(ast.NodeVisitor):
            def visit_Assign(self, node):
                # ensure the left is the var name
                if isinstance(node.targets[0], ast.Name):
                    var_name = node.targets[0].id
                    try:
                        var_value = ast.literal_eval(node.value)
                        variables[var_name] = var_value
                    except Exception as e:
                        pass

        tree = ast.parse(self.get_settings_str())
        VariableVisitor().visit(tree)
        return variables

    def get_settings_str(self) -> str:
        return self._settings_str[0]

    def set_settings_str(self, settings_str: str) -> None:
        self._settings_str[0] = settings_str

    def write(self) -> None:
        self.settings_py_path.write_text(self.get_settings_str(), encoding='utf-8')

    def add_middleware(self, new_mw: str) -> None:
        current_mws: list[str] = self.settings_const['MIDDLEWARE']
        if new_mw in current_mws:
            return
        # add middleware in order
        updated_mws = []
        for middleware in self.MIDDLEWARE_ORDER:
            if middleware == new_mw:
                updated_mws.append(new_mw)
            if middleware in current_mws:
                updated_mws.append(middleware)
        # add other middlewares not in the order list
        for middleware in current_mws:
            if middleware not in updated_mws:
                updated_mws.append(middleware)
        # add the new middleware if it's not in the order list
        if new_mw not in updated_mws:
            updated_mws.append(new_mw)
        self.settings_const['MIDDLEWARE'] = updated_mws
        self.sp.sub('MIDDLEWARE', self.sp.dump('MIDDLEWARE', updated_mws))

    def add_installed_apps(self, new_app: str) -> None:
        current_apps: list[str] = self.settings_const['INSTALLED_APPS']
        if new_app in current_apps:
            return
        current_apps.append(new_app)
        self.sp.sub('INSTALLED_APPS', self.sp.dump('INSTALLED_APPS', current_apps))



from django.contrib import admin
from django.contrib.auth.admin import UserAdmin
from .models import CustomUser
from .forms import CustomAdminUserCreationForm, CustomUserChangeForm


class CustomUserAdmin(UserAdmin):
    add_form = CustomAdminUserCreationForm
    form = CustomUserChangeForm
    model = CustomUser
    list_display = ('username', 'email', 'phone_number', 'is_staff', 'is_active')
    fieldsets = UserAdmin.fieldsets + (
        (None, {'fields': ('phone_number', 'birth_date')}),
    )
    add_fieldsets = UserAdmin.add_fieldsets + (
        (None, {'fields': ('phone_number', 'birth_date')}),
    )


admin.site.register(CustomUser, CustomUserAdmin)



class _Databases:
    name = 'DATABASES'
    new = """DATABASES = {{
    'default': {{
        'ENGINE': '{database}',
        'NAME': get_env("DB_NAME"),
        'USER': get_env("DB_USER"),
        'PASSWORD': get_env("DB_PASSWORD"),
        'HOST': get_env("DB_HOST"),
        'PORT': get_env("DB_PORT"),
    }}
}}"""


class _AsgiApplication:
    new = """ASGI_APPLICATION = '{project_name}.asgi.application'

CHANNEL_LAYERS = {{
    'default': {{
        'BACKEND': 'channels_redis.core.RedisChannelLayer',
        'CONFIG': {{
            "hosts": [(get_env('REDIS_HOST'), get_env('REDIS_PORT'))],
        }},
    }},
}}"""


class _Asgi:
    old = """\nfrom django.core.asgi import get_asgi_application

os.environ.setdefault('DJANGO_SETTINGS_MODULE', '{project_name}.settings')

application = get_asgi_application()"""

    new = """from django.core.asgi import get_asgi_application
from channels.routing import ProtocolTypeRouter, URLRouter
from channels.auth import AuthMiddlewareStack
# from some_app import routing

os.environ.setdefault('DJANGO_SETTINGS_MODULE', '{project_name}.settings')

application = ProtocolTypeRouter({{
    "http": get_asgi_application(),
    "websocket": AuthMiddlewareStack(
        URLRouter(
            routing.websocket_urlpatterns
        )
    ),
}})"""


class _AppUrls:
    new = """from django.urls import path
from .views import *

app_name = '{app_name}'

urlpatterns = [
    # path('example/', example, name='example'),
]
"""


class _Templates:
    name = 'TEMPLATES'
    new = """TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [BASE_DIR / 'templates'],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]"""


class _Languages:
    new = """LANGUAGES = [
    ('en', 'English'),
    ('zh-hans', '中文（简体）'),
]"""


class _Accounts:
    new = """LOGIN_REDIRECT_URL = '/'

LOGOUT_REDIRECT_URL = '/{app_name}/login/'

AUTH_USER_MODEL = '{app_name}.CustomUser'"""


class T:
    Databases = _Databases
    AsgiApplication = _AsgiApplication
    Asgi = _Asgi
    AppUrls = _AppUrls
    Templates = _Templates
    Languages = _Languages
    Accounts = _Accounts



import psycopg2
from psycopg2 import sql
from psplpy.other_utils import get_env
from .enums import DbBackend


def create_db(db_backend: str = DbBackend.POSTGRESQL):
    if db_backend == DbBackend.POSTGRESQL:
        conn = psycopg2.connect(
            dbname="postgres",
            user=get_env('DB_USER'),
            password=get_env('DB_PASSWORD'),
            host=get_env('DB_HOST'),
            port=get_env('DB_PORT'),
        )
        conn.autocommit = True
        cur = conn.cursor()
        cur.execute(sql.SQL("CREATE DATABASE {}").format(sql.Identifier(get_env('DB_NAME'))))
        cur.close()
        conn.close()
    else:
        raise ValueError(f'Database type {db_backend} not supported')



import shutil
from .templates import T
from .settings import *
from .utils import *
from psplpy.file_utils import get_file_paths


class DjangoInit:
    def __init__(self, project_dir: str | Path = None, database: str = None, env_file: Path | str = None,
                 show_info: bool = True):
        self.s = Settings(project_dir, show_info)

        get_env.env_file = self.s.project_dir.parent / '.env'
        if env_file:
            get_env.env_file = Path(env_file)
        self.database = database
        self.show_info = show_info
        self.rc_dir = Path(__file__).resolve().parent / 'resources'
        self._python = self._check_python()

    @staticmethod
    def _check_python() -> str:
        if os.system('python3 --version') == 0:
            return 'python3'
        return 'python'

    def _new_file(self, path: Path, content: str = None) -> None:
        if not path.exists():
            path.write_text(content, encoding='utf-8')
            self.s.sp._show_info(f'New: {content}')

    def _mkdir(self, dir_path: Path) -> None:
        if not dir_path.exists():
            dir_path.mkdir(parents=True, exist_ok=True)
            self.s.sp._show_info(f'Make dir: {dir_path}')

    def _replace_file(self, file_path: Path, old: str, new: str, not_in: str = '') -> None:
        if not file_path.exists():
            raise ValueError(f'File does not exist: {file_path}')
        content = file_path.read_text(encoding='utf-8')
        if not_in and not_in in content:
            return
        if old in content:
            file_path.write_text(content.replace(old, new), encoding='utf-8')
            self.s.sp._show_info(f'Change: {old}')
            self.s.sp._show_info(f'To: {new}')

    def asgi(self) -> 'DjangoInit':
        self.s.add_installed_apps('channels')
        self.s.sp.add_before('MIDDLEWARE', T.AsgiApplication.new.format(project_name=self.s.project_name))

        self._replace_file(self.s.project_app_dir / 'asgi.py',
                           T.Asgi.old.format(project_name=self.s.project_name),
                           T.Asgi.new.format(project_name=self.s.project_name))
        return self

    def create_db(self):
        create_db(self.database)

    def basic_init(self) -> 'DjangoInit':
        # set .env file
        s = ('from psplpy.other_utils import get_env\n'
             'get_env.env_file = Path(__file__).resolve().parent.parent.parent / ".env"')
        self.s.sp.add_before("BASE_DIR", s)
        # make global templates dir
        templates_dir = self.s.project_dir / 'templates'
        self._mkdir(templates_dir)
        self.s.sp.sub(T.Templates.name, T.Templates.new)
        shutil.copy2(self.rc_dir / 'templates' / 'base.html', templates_dir / 'base.html')
        # make global static dir
        self._mkdir(self.s.project_dir / 'static')
        self.s.sp.add_after('STATIC_URL', "STATICFILES_DIRS = [BASE_DIR / 'static',]")
        # make global media dir
        self._mkdir(self.s.project_dir / 'media')
        self.s.sp.add_before('STATIC_URL', "MEDIA_URL = '/media/'\nMEDIA_ROOT = BASE_DIR / 'media'")
        # modify allowed hosts
        self.s.sp.sub('ALLOWED_HOSTS', "ALLOWED_HOSTS = ['*']")
        # set database
        if self.database and self.database != DbBackend.SQLITE3:
            self.s.sp.sub(T.Databases.name, T.Databases.new.format(database=self.database))
        # import django.urls.include
        new = 'from django.urls import path, include'
        self._replace_file(self.s.project_app_dir / 'urls.py', 'from django.urls import path', new, not_in=new)
        return self

    def i18n(self) -> 'DjangoInit':
        self.s.add_middleware('django.middleware.locale.LocaleMiddleware')
        self.s.sp.add_after('USE_I18N', 'USE_L10N = True')
        self.s.sp.add_after('USE_TZ', "LOCALE_PATHS = [BASE_DIR / 'locale',]")
        self.s.sp.add_before('LANGUAGE_CODE', T.Languages.new)
        new = "path('i18n/', include('django.conf.urls.i18n'))"
        self._replace_file(self.s.project_app_dir / 'urls.py', ',\n]',
                           f",\n    {new},\n]", not_in=new)
        return self

    def startapp(self, name: str) -> 'DjangoInit':
        os.system(f'{self._python} manage.py startapp {name}')
        self.s.add_installed_apps(name)
        # modify project urls.py
        new = f"path('{name}/', include('{name}.urls', namespace='{name}'))"
        self._replace_file(self.s.project_app_dir / 'urls.py', ',\n]',
                           f",\n    {new},\n]", not_in=new)
        app_path = self.s.project_dir / name
        # create urls.py
        self._new_file(app_path / 'urls.py', T.AppUrls.new.format(app_name=name))
        # create commands dir
        self._mkdir(app_path / 'management' / 'commands')
        return self

    def _cp_dir(self, src_dir: Path, dst_dir: Path) -> None:
        dst_dir.mkdir(parents=True, exist_ok=True)

        rel_src_paths = get_file_paths(src_dir, relative=True, generator=True)
        for rel_src_path in rel_src_paths:
            src_path = src_dir / rel_src_path
            dst_path = dst_dir / rel_src_path
            dst_path.parent.mkdir(parents=True, exist_ok=True)
            shutil.copy2(src_path, dst_path)
            self.s.sp._show_info(f'Copy file: {src_path} to {dst_path}')

    def accounts_app(self) -> 'DjangoInit':
        app_name = 'accounts'
        self.startapp(app_name)
        self.s.sp.add_after("AUTH_PASSWORD_VALIDATORS", T.Accounts.new.format(app_name=app_name))
        self._cp_dir(self.rc_dir / 'templates' / 'registration', self.s.project_dir / 'templates' / 'registration')
        self._cp_dir(self.rc_dir / 'apps' / 'accounts', self.s.project_dir / app_name)
        return self
